@@ -5,15 +5,13 @@ import re # Added for markdown formatting
 import sys
 import traceback
 from pathlib import Path
-from typing import List, Set, Dict, Optional
-import atexit
+from typing import List, Set, Dict, Optional, Tuple
 
-# readline is not available on all platforms (e.g., standard Windows cmd)
-try:
-    import readline
-    READLINE_AVAILABLE = True
-except ImportError:
-    READLINE_AVAILABLE = False
+from prompt_toolkit import PromptSession, print_formatted_text
+from prompt_toolkit.completion import Completer
+from prompt_toolkit.formatted_text import FormattedText
+from prompt_toolkit.history import FileHistory
+from prompt_toolkit.styles import Style
 
 from tinycoder.chat_history import ChatHistoryManager
 from tinycoder.code_applier import CodeApplier
@@ -30,8 +28,8 @@ from tinycoder.repo_map import RepoMap
 from tinycoder.rule_manager import RuleManager
 from tinycoder.shell_executor import ShellExecutor
 from tinycoder.input_preprocessor import InputPreprocessor
-from tinycoder.ui.console_interface import ring_bell, ConsoleInterface, prompt_user_input
-from tinycoder.ui.command_completer import CommandCompleter, READLINE_AVAILABLE as COMPLETION_READLINE_AVAILABLE
+from tinycoder.ui.console_interface import ring_bell, prompt_user_input
+from tinycoder.ui.command_completer import PTKCommandCompleter
 from tinycoder.ui.log_formatter import ColorLogFormatter, STYLES, COLORS as FmtColors, RESET
 from tinycoder.ui.spinner import Spinner
 from tinycoder.docker_manager import DockerManager
@@ -54,10 +52,10 @@ class App:
         self._init_prompt_builder()
         self._setup_rules_manager()
         self._init_input_preprocessor() # Initialize InputPreprocessor
-        self._init_console_interface() # Initialize ConsoleInterface
+        self._init_prompt_session() # Initialize PromptSession and styles
+        self._reconfigure_logging_for_ptk() # Switch to prompt_toolkit-aware logging
         self._init_app_state()
         self._init_command_handler()
-        self._configure_readline()
         self._init_app_components()
         self._log_final_status()
         self._add_initial_files(files)
@@ -213,6 +211,92 @@ class App:
         )
         self.logger.debug("RuleManager initialized.")
 
+    def _reconfigure_logging_for_ptk(self) -> None:
+        """
+        Swaps the initial StreamHandler with a prompt_toolkit-based handler
+        to ensure logging integrates smoothly with the prompt session.
+        This is called after startup messages are logged.
+        """
+        root_logger = logging.getLogger()
+        old_handler = None
+        
+        # Find the existing stream handler
+        for handler in root_logger.handlers[:]:
+            if isinstance(handler, logging.StreamHandler):
+                old_handler = handler
+                break
+        
+        if not old_handler:
+            self.logger.debug("No existing StreamHandler found to replace. Skipping log reconfiguration.")
+            return
+
+        # Create the new handler and give it the same formatter and level as the old one
+        from tinycoder.ui.log_formatter import PromptToolkitLogHandler
+        ptk_handler = PromptToolkitLogHandler(self.style)
+        ptk_handler.setFormatter(old_handler.formatter)
+        ptk_handler.setLevel(old_handler.level)
+        
+        # Replace the old handler with the new one
+        root_logger.removeHandler(old_handler)
+        root_logger.addHandler(ptk_handler)
+
+        self.logger.debug("Logging reconfigured to use PromptToolkitLogHandler.")
+
+    def _init_prompt_session(self) -> None:
+        """Initializes the prompt_toolkit session, completer, and style."""
+        # Setup history file
+        hist_dir = Path.home() / ".local" / "share" / APP_NAME
+        hist_dir.mkdir(parents=True, exist_ok=True)
+        history_file = hist_dir / HISTORY_FILE
+
+        # Setup completer
+        self.completer: Optional[Completer] = PTKCommandCompleter(self.file_manager, self.git_manager)
+
+        self.prompt_session = PromptSession(
+            history=FileHistory(str(history_file)),
+            completer=self.completer,
+            multiline=True,
+            prompt_continuation="... "
+        )
+        self.logger.debug("Prompt session initialized with history and completer.")
+
+        # Central style for the application
+        self.style = Style.from_dict({
+            # Prompt
+            'prompt.mode': 'bold fg:ansigreen',
+            'prompt.separator': 'fg:ansibrightblack',
+            'rprompt.tokens.low': 'fg:ansigreen',
+            'rprompt.tokens.medium': 'fg:ansiyellow',
+            'rprompt.tokens.high': 'fg:ansired',
+            'rprompt.text': 'fg:ansibrightblack',
+            # Bottom Toolbar
+            'bottom-toolbar':          'bg:#222222 fg:#aaaaaa',      # Base style for the toolbar, single background
+            'bottom-toolbar.low':      'bg:#222222 fg:ansigreen bold', # Total tokens (low)
+            'bottom-toolbar.medium':   'bg:#222222 fg:ansiyellow bold', # Total tokens (medium)
+            'bottom-toolbar.high':     'bg:#222222 fg:ansired bold',   # Total tokens (high)
+            # Assistant & Markdown
+            'assistant.header': 'bold fg:ansicyan',
+            'markdown.h1': 'bold fg:ansiblue',
+            'markdown.h2': 'bold fg:ansimagenta',
+            'markdown.h3': 'bold fg:ansicyan',
+            'markdown.bold': 'bold',
+            'markdown.code': 'fg:ansiyellow',
+            'markdown.code-block': 'fg:ansigreen',
+            'markdown.list': 'fg:ansicyan',
+            # Diffs
+            'diff.header': 'bold',
+            'diff.plus': 'fg:ansigreen',
+            'diff.minus': 'fg:ansired',
+            # Logging
+            'log.debug': 'fg:#888888',
+            'log.info': '', # Default terminal color
+            'log.warning': 'fg:ansiyellow',
+            'log.error': 'fg:ansired',
+            'log.critical': 'bold fg:ansired',
+        })
+        self.logger.debug("Application style defined.")
+
+
     def _init_input_preprocessor(self) -> None:
         """Initializes the InputPreprocessor."""
         self.input_preprocessor = InputPreprocessor(
@@ -223,16 +307,6 @@ class App:
         )
         self.logger.debug("InputPreprocessor initialized.")
 
-    def _init_console_interface(self) -> None:
-        """Initializes the ConsoleInterface."""
-        self.console_interface = ConsoleInterface(
-            logger=self.logger,
-            get_app_mode_func=lambda: self.mode, # Pass a callable to get current mode
-            readline_available=READLINE_AVAILABLE, # Pass the app-level constant
-            get_token_breakdown_func=self._get_current_context_token_breakdown # Pass token breakdown function
-        )
-        self.logger.debug("ConsoleInterface initialized.")
-
     def _init_app_state(self) -> None:
         """Initializes basic application state variables."""
         self.coder_commits: Set[str] = set()
@@ -242,6 +316,7 @@ class App:
         self.include_repo_map: bool = True # Default to including the repo map
         self.total_input_tokens: int = 0
         self.total_output_tokens: int = 0
+        self.cached_token_breakdown: Dict[str, int] = {} # For UI performance
         self.logger.debug("Basic app state initialized (commits, mode, lint status, repo map toggle, usage tracking).")
 
     def toggle_repo_map(self, state: bool) -> None:
@@ -357,12 +432,13 @@ class App:
         self.logger.debug("CommandHandler initialized.")
 
     def _init_app_components(self) -> None:
-        """Initializes EditParser, CodeApplier, and determines input function."""
+        """Initializes EditParser, CodeApplier, and ShellExecutor."""
         self.edit_parser = EditParser()
         self.code_applier = CodeApplier(
             file_manager=self.file_manager,
             git_manager=self.git_manager,
-            input_func=prompt_user_input, # Use centralized input for confirmations within applier
+            input_func=self._prompt_for_confirmation, # Use centralized input for confirmations
+            style=self.style, # Pass style for diff printing
         )
         # Initialize ShellExecutor
         self.shell_executor = ShellExecutor(
@@ -370,9 +446,7 @@ class App:
             history_manager=self.history_manager,
             git_root=self.git_root
         )
-        # Determine primary input function for main loop using ConsoleInterface
-        self.input_func = self.console_interface.determine_input_function()
-        self.logger.debug("App components (Parser, Applier, ShellExecutor, Input Func from ConsoleInterface) initialized.")
+        self.logger.debug("App components (Parser, Applier, ShellExecutor) initialized.")
 
     def _handle_docker_automation(self, modified_files_rel: List[str], non_interactive: bool = False):
         """
@@ -552,85 +626,54 @@ class App:
             return str(Path(self.git_root).resolve())
         else:
             return str(Path.cwd().resolve())
+
+    async def _prompt_for_confirmation(self, prompt_text: str) -> str:
+        """Async user prompt for confirmations within the main app loop."""
+        ring_bell()
+        # Use prompt_async for async contexts
+        response = await self.prompt_session.prompt_async(prompt_text)
+        # Strip any escape sequences and control characters
+        # This handles cases where Alt+Enter or other key combos add unwanted characters
+        import re
+        cleaned_response = re.sub(r'[\x00-\x1f\x7f-\x9f]', '', response)
+        return cleaned_response.strip()
+
+
+    def _get_bottom_toolbar_tokens(self) -> FormattedText:
+        """
+        Generates the formatted text for the bottom toolbar from cached token context.
+        This function must be extremely fast as it's called on every redraw.
+        """
+        breakdown = self.cached_token_breakdown
+        total = breakdown.get("total", 0)
         
-    def _configure_readline(self):
-        """Configures readline for history and command completion if available."""
-        # Use the local READLINE_AVAILABLE for history, and the imported one for completer
-        if not READLINE_AVAILABLE: # This is the one defined in __init__.py for history
-            self.logger.info("Readline module not available for history features.")
-            # No return here yet, completion might still be attempted if COMPLETION_READLINE_AVAILABLE is true
-
-        if not COMPLETION_READLINE_AVAILABLE: # This is from command_completer.py
-            self.logger.info("Readline module not available for command completion features.")
+        # Determine color based on total tokens
+        total_color_class = 'class:bottom-toolbar.low'
+        if total > 25000:
+            total_color_class = 'class:bottom-toolbar.high'
+        elif total > 15000:
+            total_color_class = 'class:bottom-toolbar.medium'
         
-        if not READLINE_AVAILABLE and not COMPLETION_READLINE_AVAILABLE:
-            self.logger.info("Readline module not available. Skipping history and completion setup.")
-            return
+        # Full word descriptions, single style for most text
+        parts = [
+            ('class:bottom-toolbar', '  Context: '),
+            (total_color_class, f'{total:,}'),
+            ('class:bottom-toolbar', f" (Prompt: {breakdown.get('prompt_rules', 0):,} | "),
+            ('class:bottom-toolbar', f"Map: {breakdown.get('repo_map', 0):,} | "),
+            ('class:bottom-toolbar', f"Files: {breakdown.get('files', 0):,} | "),
+            ('class:bottom-toolbar', f"History: {breakdown.get('history', 0):,})  "),
+        ]
 
-        self.logger.debug("Readline available. Configuring...")
-
-        # --- Completion Setup ---
-        if COMPLETION_READLINE_AVAILABLE:
-            try:
-                completer_instance = CommandCompleter(self.file_manager, self.git_manager)
-                readline.set_completer(completer_instance.complete)
-
-                # Set delimiters for completion. Crucially, DO NOT include path separators like '/' or '.'
-                # if you want to complete segments containing them. Let's stick to whitespace and typical shell separators.
-                # Space is the most important delimiter here to separate `/add` from the path.
-                readline.set_completer_delims(' \t\n`~!@#$%^&*()=+[{]}|;:\'",<>?') # Removed \ . /
-
-                # Configure Tab key binding
-                if 'libedit' in readline.__doc__: # macOS/libedit
-                    readline.parse_and_bind("bind -e") # Ensure emacs mode
-                    readline.parse_and_bind("bind '\t' rl_complete")
-                    self.logger.debug("Using libedit Tab binding.")
-                else: # GNU readline
-                    readline.parse_and_bind("tab: complete")
-                    self.logger.debug("Using standard readline Tab binding.")
-
-            except Exception as e:
-                self.logger.error(f"Failed to configure readline completion: {e}", exc_info=True)
-
-
-        # --- History Setup ---
-        if READLINE_AVAILABLE: # Guard history setup with the local READLINE_AVAILABLE
-            # Use project identifier for potentially project-specific history
-            # Or use a generic one in the user's home directory
-            hist_dir = Path.home() / ".local" / "share" / APP_NAME
-            hist_dir.mkdir(parents=True, exist_ok=True) # Ensure directory exists
-            # Use a consistent history filename
-            self.history_file = hist_dir / HISTORY_FILE
-
-            try:
-                # Set history length (optional)
-                readline.set_history_length(1000)
-                # Read history file *after* setting bindings and completer, *before* registering save
-                if self.history_file.exists():
-                    readline.read_history_file(self.history_file)
-                    self.logger.debug(f"Read history from {self.history_file}")
-                else:
-                    self.logger.debug(f"History file {self.history_file} not found, starting fresh.")
-
-                # Register saving history on exit
-                atexit.register(self._save_readline_history)
-                self.logger.debug("Readline history configured and loaded.")
-            except Exception as e:
-                self.logger.error(f"Failed to configure readline history: {e}", exc_info=True)
-        else:
-            self.logger.debug("Skipping readline history setup as readline is not available.")
-
-
-    def _save_readline_history(self):
-        """Saves the readline history to the designated file."""
-        if not READLINE_AVAILABLE: # Use local READLINE_AVAILABLE
-            return
-        try:
-            readline.write_history_file(self.history_file)
-            self.logger.debug(f"Readline history saved to {self.history_file}")
-        except Exception as e:
-            self.logger.error(f"Failed to save readline history to {self.history_file}: {e}")
+        return FormattedText(parts)
 
+    def _update_and_cache_token_breakdown(self) -> None:
+        """
+        Performs the expensive token calculation and caches the result.
+        This should only be called when the context has actually changed.
+        """
+        self.cached_token_breakdown = self._get_current_context_token_breakdown()
+        self.logger.debug("Token context breakdown cache updated.")
+        
     def _get_current_context_token_breakdown(self) -> Dict[str, int]:
         """Calculates the approximate token count breakdown for the current context."""
         # This is a helper function to estimate tokens from characters.
@@ -770,13 +813,19 @@ class App:
                 )
                 return None
             else:
+                assistant_header = [('class:assistant.header', 'ASSISTANT'), ('', ':\n')]
+                print_formatted_text(FormattedText(assistant_header), style=self.style)
+
                 # Format for display if in ask mode and not an edit block
                 if self.mode == "ask" and response_content and not response_content.strip().startswith("<"):
-                    display_response = self._format_markdown_for_terminal(response_content)
-                    self.logger.info(f"\n{FmtColors['CYAN']}{STYLES['BOLD']}ASSISTANT{RESET}:\n" + display_response + "\n")
+                    display_response_tuples = self._format_markdown_for_terminal(response_content)
+                    print_formatted_text(FormattedText(display_response_tuples), style=self.style)
                 else:
-                    self.logger.info(f"\n{FmtColors['CYAN']}{STYLES['BOLD']}ASSISTANT{RESET}:\n" + response_content + "\n")
+                    # Print raw content, but use prompt_toolkit to handle potential long lines
+                    print_formatted_text(response_content)
                 
+                print() # Add a final newline for spacing
+
                 output_chars = len(response_content)
                 output_tokens = round(output_chars / 4) # Based on raw response
                 self.total_output_tokens += output_tokens
@@ -877,7 +926,7 @@ class App:
                 "tool", f"Undid commit {last_hash}"
             )
 
-    def _handle_llm_file_requests(self, requested_files_from_llm: List[str]) -> bool:
+    async def _handle_llm_file_requests(self, requested_files_from_llm: List[str]) -> bool:
         """
         Handles LLM's request for additional files.
         Checks existence, prompts user, adds files, and sets up reflection.
@@ -935,7 +984,7 @@ class App:
             self.logger.info(f"  {i+1}. {FmtColors['CYAN']}{fname}{RESET}")
         
         confirm_prompt = "Add these files to context? (y/N, or list indices like '1,3'): "
-        confirm = prompt_user_input(confirm_prompt).strip().lower()
+        confirm = (await self._prompt_for_confirmation(confirm_prompt)).strip().lower()
 
         if not confirm: # Handles cancellation from prompt_user_input
             self.logger.info("\nFile addition (from LLM request) cancelled by user.")
@@ -1039,7 +1088,7 @@ class App:
         # Use print() to ensure the summary is always visible regardless of log level
         print(summary_message)
 
-    def process_user_input(self, non_interactive: bool = False):
+    async def process_user_input(self, non_interactive: bool = False):
         """Processes the latest user input (already in history), sends to LLM, handles response."""
         response = self._send_to_llm()
 
@@ -1053,7 +1102,7 @@ class App:
 
             # --- Handle File Requests First ---
             if requested_files:
-                if self._handle_llm_file_requests(requested_files):
+                if await self._handle_llm_file_requests(requested_files):
                     # A reflection message is set (e.g., files added, user cancelled).
                     # The run_one loop will pick this up. We are done for this turn.
                     return 
@@ -1064,7 +1113,7 @@ class App:
             if not self.reflected_message and self.mode == "code":
                 if edits:
                     all_succeeded, failed_indices, modified_files, lint_errors = (
-                        self.code_applier.apply_edits(edits)
+                        await self.code_applier.apply_edits(edits)
                     )
                     self.lint_errors_found = lint_errors 
 
@@ -1100,7 +1149,7 @@ class App:
                     combined_errors = "\n".join(error_messages)
                     self.logger.error(combined_errors)
 
-                    fix_lint = prompt_user_input("Attempt to fix lint errors? (y/N): ")
+                    fix_lint = await self._prompt_for_confirmation("Attempt to fix lint errors? (y/N): ")
                     if fix_lint.lower() == "y":
                         self.reflected_message = combined_errors
             
@@ -1170,74 +1219,38 @@ class App:
         self.lint_errors_found = {}
         self.reflected_message = None
 
-    def _format_markdown_for_terminal(self, markdown_text: str) -> str:
-        """Applies simple ANSI styling to markdown text for terminal readability."""
-        processed_lines = []
+    def _format_markdown_for_terminal(self, markdown_text: str) -> List[Tuple[str, str]]:
+        """Converts markdown text to a list of (style_class, text) tuples for prompt_toolkit."""
+        formatted_text = []
         in_code_block = False
-        # Code block language is not used yet, but could be for syntax highlighting in future
-        # code_block_lang = "" 
 
         for line in markdown_text.splitlines():
-            stripped_line = line.lstrip() # Use lstrip for checking prefixes, keep original indent for code blocks
+            stripped_line = line.lstrip()
 
-            # Handle Fenced Code Blocks
             if stripped_line.startswith("```"):
-                if in_code_block:
-                    in_code_block = False
-                    # Color the fence itself
-                    processed_lines.append(FmtColors['GREY'] + line + RESET)
-                else:
-                    in_code_block = True
-                    # code_block_lang = stripped_line[3:].strip()
-                    # lang_display = f" ({code_block_lang})" if code_block_lang else ""
-                    processed_lines.append(FmtColors['GREY'] + line + RESET)
+                in_code_block = not in_code_block
+                formatted_text.append(('class:markdown.code-block', line + '\n'))
                 continue
 
             if in_code_block:
-                # Preserve original line including leading spaces for indentation
-                processed_lines.append(FmtColors['GREEN'] + line + RESET)
+                formatted_text.append(('class:markdown.code-block', line + '\n'))
                 continue
 
-            # Handle Headers (e.g., # H1, ## H2)
             if stripped_line.startswith("#"):
-                level = 0
-                temp_line = stripped_line
-                while temp_line.startswith('#'):
-                    level += 1
-                    temp_line = temp_line[1:]
-                
-                if temp_line.startswith(' '): # Make sure it's a header, not just a line starting with #
-                    header_text = temp_line[1:]
-                    original_indent = line[:-len(stripped_line)] # Get leading whitespace
-                    if level == 1: # H1
-                        line = f"{original_indent}{STYLES['BOLD']}{FmtColors['BLUE']}# {header_text}{RESET}"
-                    elif level == 2: # H2
-                        line = f"{original_indent}{STYLES['BOLD']}{FmtColors['MAGENTA']}## {header_text}{RESET}"
-                    else: # H3+
-                        line = f"{original_indent}{STYLES['BOLD']}{FmtColors['CYAN']}{'#' * level} {header_text}{RESET}"
-            else:
-                # Handle Bold (**text** or __text__)
-                # Apply bold and then immediately reset. The reset also turns off previous colors if any.
-                # If we want to keep surrounding colors, this would need to be more complex.
-                # Using a non-greedy match (.*?)
-                line = re.sub(r'\*\*(.*?)\*\*', rf'{STYLES["BOLD"]}\1{RESET}', line)
-                line = re.sub(r'__(.*?)__', rf'{STYLES["BOLD"]}\1{RESET}', line) # STYLES.NORMAL might be needed if terminal holds bold
-
-                # Handle Inline Code (`code`) - apply after bold to allow `**bold code**`
-                # (though typical markdown `**`code`**` would be `code` bolded)
-                # For simplicity, this colors the content of `...`
-                line = re.sub(r'`(.*?)`', rf'{FmtColors["YELLOW"]}\1{RESET}', line)
-                
-                # Handle Lists (* item, - item, 1. item)
-                # Simple coloring of the marker. Preserves leading whitespace.
-                line = re.sub(r'^(\s*)(\* |\- |\+ )', rf'\1{FmtColors["CYAN"]}\2{RESET}', line, 1)
-                line = re.sub(r'^(\s*)([0-9]+\. )', rf'\1{FmtColors["CYAN"]}\2{RESET}', line, 1)
+                level = len(stripped_line) - len(stripped_line.lstrip('#'))
+                if stripped_line[level:].startswith(' '):
+                    style_class = f'class:markdown.h{min(level, 3)}'
+                    formatted_text.append((style_class, line + '\n'))
+                    continue
             
-            processed_lines.append(line)
+            # This part is a simplification. A real markdown parser would be needed for complex cases.
+            # For now, we just append the line. Further regex for bold/italic could be added here.
+            formatted_text.append(('', line + '\n'))
 
-        return "\n".join(processed_lines)
+        return formatted_text
 
-    def _handle_command(self, user_message: str) -> bool:
+
+    async def _handle_command(self, user_message: str) -> bool:
         """
         Handles a command input. Returns False if the command is to exit, True otherwise.
         May modify self.mode.
@@ -1251,12 +1264,12 @@ class App:
         if prompt_arg:
             # If command included a prompt (e.g., /ask "What?"), process it *now*
             # Don't preprocess command arguments (e.g., URL check)
-            if not self.run_one(prompt_arg, preproc=False):
+            if not await self.run_one(prompt_arg, preproc=False):
                 return False  # Exit signal from processing the prompt
 
         return True  # Continue processing
 
-    def run_one(self, user_message, preproc, non_interactive=False):
+    async def run_one(self, user_message, preproc, non_interactive=False):
         """
         Processes a single user message, including potential reflection loops in interactive mode.
 
@@ -1269,7 +1282,7 @@ class App:
 
         if preproc:
             if user_message.startswith("/"):
-                if not self._handle_command(user_message):
+                if not await self._handle_command(user_message):
                     return False  # Exit signal
                 else:
                     return True  # Command handled, stop further processing for this input cycle
@@ -1327,7 +1340,7 @@ class App:
 
         # Initial processing of the user message
         self.history_manager.add_message("user", message)  # Use history manager
-        self.process_user_input(non_interactive=non_interactive)  # This now handles LLM call, edits, linting
+        await self.process_user_input(non_interactive=non_interactive)  # This now handles LLM call, edits, linting
 
         # Check if reflection is needed *and* allowed (interactive mode)
         while not non_interactive and self.reflected_message:
@@ -1351,125 +1364,59 @@ class App:
 
             # Add the reflected message to history *before* processing
             self.history_manager.add_message("user", message)
-            self.process_user_input(non_interactive=non_interactive)  # Process the reflected input
+            await self.process_user_input(non_interactive=non_interactive)  # Process the reflected input
 
         return True  # Indicate normal processing occurred (or finished reflection loop)
 
-    def run(self):
-        """Main loop for the chat application."""
-        # Determine repo map status string with more detail
-        repo_map_status_output_str: str
-        if self.include_repo_map:
-            user_exclusions = self.repo_map.get_user_exclusions()
-            if user_exclusions:
-                num_exclusions = len(user_exclusions)
-                exclusions_details = f"with {num_exclusions} exclusion"
-                if num_exclusions > 1:
-                    exclusions_details += "s"
-                # Using YELLOW for enabled with exclusions
-                repo_map_status_output_str = f"{FmtColors['YELLOW']}{STYLES['BOLD']}Enabled ({exclusions_details}){RESET}"
-            else:
-                # Using GREEN for standard enabled (no exclusions)
-                repo_map_status_output_str = f"{FmtColors['GREEN']}{STYLES['BOLD']}Enabled{RESET}"
-        else:
-            # Using default terminal color (bolded) for disabled
-            repo_map_status_output_str = f"{STYLES['BOLD']}Disabled{RESET}"
-        
-        # Use FmtColors and STYLES for the welcome message
-        # Apply specific color (GREEN) before BOLD, then RESET immediately after.
-        # The rest of the message will use the default INFO format (terminal default color).
-        
-        # Constructing the welcome message parts
-        model_line = f"  Model: {FmtColors['GREEN']}{STYLES['BOLD']}{self.model}{RESET}"
+    async def run(self):
+        """Main loop for the chat application using prompt_toolkit."""
+        # Initial token calculation before the first prompt
+        self._update_and_cache_token_breakdown()
 
-        # Active Rules status
-        num_active_rules = len(self.rule_manager.last_loaded_rule_names)
-        active_rules_names_sorted = sorted(list(self.rule_manager.last_loaded_rule_names))
-        
-        active_rules_status_output_str: str
-        if num_active_rules > 0:
-            max_names_to_show = 2 # Be concise for welcome message
-            names_str = ", ".join(active_rules_names_sorted[:max_names_to_show])
-            if num_active_rules > max_names_to_show:
-                names_str += f", ... ({num_active_rules - max_names_to_show} more)"
-            
-            active_rules_status_output_str = f"{FmtColors['GREEN']}{STYLES['BOLD']}Enabled ({num_active_rules}){RESET} [{names_str}]"
-        else:
-            # Using default terminal color (bolded) for none
-            active_rules_status_output_str = f"{STYLES['BOLD']}None{RESET}"
-        
-        active_rules_line = f"  Rules: {active_rules_status_output_str}"
-        help_line = f"  Type {FmtColors['CYAN']}{STYLES['BOLD']}/help{RESET} for commands, or {FmtColors['CYAN']}{STYLES['BOLD']}!<cmd>{RESET} to run shell commands. \n"
+        # Use logger for startup info, which has its own color formatting.
+        self.logger.info(f"  Model: {FmtColors['GREEN']}{STYLES['BOLD']}{self.model}{RESET}")
+        self.logger.info("  Type /help for commands, or !<cmd> to run shell commands.\n")
 
-        # Determine Docker status string
-        docker_status_output_str: str
-        if self.docker_manager and self.docker_manager.is_available:
-            if self.docker_manager.compose_file:
-                # Attempt to get a relative path for the compose file for cleaner display
-                compose_file_display_path = str(self.docker_manager.compose_file)
-                if self.docker_manager.root_dir and self.docker_manager.compose_file.is_relative_to(self.docker_manager.root_dir):
-                    compose_file_display_path = str(self.docker_manager.compose_file.relative_to(self.docker_manager.root_dir))
-                
-                docker_status_output_str = f"{FmtColors['GREEN']}{STYLES['BOLD']}Enabled{RESET} (compose: {FmtColors['CYAN']}{compose_file_display_path}{RESET})"
-            else:
-                docker_status_output_str = f"{FmtColors['YELLOW']}{STYLES['BOLD']}Enabled (no compose file detected){RESET}"
-        else:
-            docker_status_output_str = f"{STYLES['BOLD']}Disabled{RESET}"
-        
-        docker_line = f"  Docker: {docker_status_output_str}"
-
-        # Log each part on a new line for clarity
-        self.logger.info(model_line)
-        self.logger.info(docker_line) # Added Docker status line
-        self.logger.info(active_rules_line)
-        self.logger.info(help_line)
-        
-        ctrl_c_pressed_once = False # Initialize flag outside the loop
         while True:
             try:
-                # DO NOT reset ctrl_c_pressed_once = False here anymore
+                # 1. Build the prompt message
+                mode_str = self.mode
+                prompt_message = FormattedText([
+                    ('class:prompt.mode', f'({mode_str})'),
+                    ('class:prompt.separator', ' > '),
+                ])
 
-                ring_bell()  # Ring the bell before input
-                inp = self.input_func() # This function handles its own KeyboardInterrupt by returning None
+                # 2. Build the bottom toolbar with token info
+                bottom_toolbar = self._get_bottom_toolbar_tokens
 
-                if inp is None:
-                    # Input was cancelled (Ctrl+C in readline loop or stdin read)
-                    if ctrl_c_pressed_once:
-                        print("\nExiting.", file=sys.stderr)
-                        break # Exit on second consecutive Ctrl+C during input
-                    else:
-                        ctrl_c_pressed_once = True
-                        print("\nInput cancelled. Press Ctrl+C again to exit.", file=sys.stderr)
-                        # Continue the loop to prompt for input again
-                        continue # Go to next loop iteration
+                # 3. Get input from the user
+                ring_bell()
+                inp = await self.prompt_session.prompt_async(
+                    prompt_message,
+                    bottom_toolbar=bottom_toolbar,
+                    style=self.style
+                )
 
-                # --- If we get here, input was successful (not None) ---
-                # Reset the flag *after* successful input, before processing
-                ctrl_c_pressed_once = False
-
-                # Strip leading/trailing whitespace
+                # 4. Process the input
                 processed_inp = inp.strip()
                 if not processed_inp:
-                    continue  # Skip empty input
-
-                # Process the valid input
-                status = self.run_one(processed_inp, preproc=True)
-                if not status:
-                    break  # Exit signal from run_one (e.g., /exit command)
-
-            except KeyboardInterrupt:  # Handle Ctrl+C pressed *outside* the input function
-                self.spinner.stop()  # Ensure spinner is stopped on any interrupt that reaches here
-                if ctrl_c_pressed_once:
-                    print("\nExiting.", file=sys.stderr)
-                    break # Exit on second consecutive Ctrl+C (one outside input, one before/during)
-                else:
-                    ctrl_c_pressed_once = True
-                    print("\nOperation interrupted. Press Ctrl+C again to exit.", file=sys.stderr)
-                    # Continue the loop to prompt for input again
                     continue
-            except EOFError:  # Handle Ctrl+D (still treated as exit)
+
+                status = await self.run_one(processed_inp, preproc=True)
+                if not status:
+                    break # Exit signal from run_one (e.g., /exit command)
+
+                # 5. Update the token cache for the *next* prompt render.
+                self._update_and_cache_token_breakdown()
+
+            except KeyboardInterrupt:
+                # User pressed Ctrl+C at the prompt.
+                # This will cancel the current input and prompt again.
+                continue
+            except EOFError:
+                # User pressed Ctrl+D.
                 print("\nExiting (EOF).", file=sys.stderr)
-                break  # Exit on Ctrl+D
+                break
 
         self._display_usage_summary()
         self.logger.info("Goodbye! 👋")
