@@ -4,7 +4,6 @@ from pathlib import Path # Added for globbing
 from typing import TYPE_CHECKING, Callable, Optional, Tuple
 
 from tinycoder.unittest_runner import run_tests
-from tinycoder.editor import launch_editor_cli
 
 if TYPE_CHECKING:
     from tinycoder.file_manager import FileManager
@@ -436,35 +435,6 @@ class CommandHandler:
                 self.logger.info("  Use: on, off, show, exclude <pattern>, include <pattern>, list_exclusions")
             return True, None
 
-        elif command == "/edit":
-            filenames = re.findall(r"\"(.+?)\"|(\S+)", args_str)
-            filenames = [name for sublist in filenames for name in sublist if name]
-
-            if not filenames or len(filenames) > 1:
-                self.logger.error('Usage: /edit <filename_or_path>')
-                self.logger.info('Example: /edit "my file.py"  OR  /edit path/to/file.txt')
-                return True, None
-            
-            fname_to_edit = filenames[0]
-            abs_path = self.file_manager.get_abs_path(fname_to_edit)
-
-            if not abs_path:
-                # get_abs_path logs an error if path is invalid or outside scope
-                return True, None
-
-            self.logger.info(f"Starting built-in editor for {abs_path}...")
-            try:
-                launch_editor_cli(str(abs_path))
-                self.logger.info(f"Editor session for {abs_path} finished.")
-                self.write_history_func("tool", f"Finished editing {fname_to_edit} using built-in editor.")
-                # Optional: If file was modified and in context, you might want to indicate it
-                # or prompt user to re-add for LLM to see changes immediately.
-                # For now, user can manually re-add or mention file.
-            except Exception as e:
-                self.logger.error(f"The built-in editor encountered an error for {abs_path}: {e}")
-                # For more detailed debugging, consider logging traceback if needed:
-                # import traceback
-                # self.logger.debug(traceback.format_exc())
             return True, None
 
         elif command == "/help":
@@ -480,7 +450,6 @@ class CommandHandler:
   /undo                       Undo the last commit made by {self.app_name}.
   /ask                        Switch to ASK mode (answer questions, no edits).
   /code                       Switch to CODE mode (make edits).
-  /edit <filename>            Open the specified file in a built-in text editor.
   /tests                      Run unit tests (runs in container if docker-compose.yml is present).
   /docker ps                  Show status of Docker containers.
   /docker logs <service>      Stream logs from a Docker container.
