@@ -169,19 +169,27 @@ class DockerManager:
                 if ':' not in line:
                     break  # End of dict block (not a key:value pair)
 
-                key, value_str = line.split(':', 1)
+                key, value_str_raw = line.split(':', 1)
                 key = key.strip()
-                value_str = value_str.strip()
+                value_str = value_str_raw.strip() # Stripped value for logic checks
 
-                if value_str:
-                    node[key] = value_str
+                # Attempt to parse children first.
+                # A child node starts at current_index + 1 and must be more indented than parent_indent (first_indent here).
+                child_node, next_index_after_child = self._build_node(lines, current_index + 1, first_indent)
+
+                if child_node is not None:
+                    node[key] = child_node
+                    current_index = next_index_after_child
+                elif value_str_raw: # If there was anything after ':', even just whitespace (now stripped to value_str)
+                                    # This covers "key: value" and "key: " (value_str is empty) and "key:" (value_str is empty)
+                                    # The critical part is that value_str_raw was not empty, meaning ':' was not at line end.
+                                    # If value_str_raw.strip() is non-empty, use it. Otherwise, if value_str_raw existed (e.g. "key: "), use ""
+                    node[key] = value_str # Assign the stripped value, which could be ""
+                    current_index += 1
+                else: # No child node, and value_str_raw was empty (e.g. "key:" at end of line, or "key:\n")
+                      # This implies ':' was the last non-whitespace char or line ended after ':'.
+                    node[key] = None # Standard representation for an empty YAML value
                     current_index += 1
-                else:
-                    # The value is a nested structure on subsequent lines.
-                    child_node, next_index = self._build_node(lines, current_index + 1, first_indent)
-                    if child_node is not None:
-                        node[key] = child_node
-                    current_index = next_index
             return node, current_index
 
     def find_affected_services(self, modified_files: List[Path]) -> Dict[str, Set[str]]:
@@ -293,16 +301,43 @@ class DockerManager:
         else:
             self.logger.info(f"Service '{service_name}' restarted successfully.")
 
+    def up_service_recreate(self, service_name: str) -> bool:
+        """
+        Brings up a service, forcing recreation of its containers, using the latest image.
+        Assumes the image has already been built if necessary.
+        """
+        self.logger.info(f"Recreating service '{service_name}' with the latest image...")
+        # --no-build: Assumes build was handled separately
+        # --force-recreate: Ensures old container is replaced
+        # -d: Detached mode
+        command = ['docker', 'compose', 'up', '-d', '--force-recreate', '--no-build', service_name]
+        success, stdout, stderr = self._run_command(command)
+        if not success:
+            self.logger.error(f"Failed to recreate service '{service_name}':\n{stderr}")
+            if stdout: # Sometimes error details are in stdout for 'up'
+                self.logger.error(f"Stdout:\n{stdout}")
+            return False
+        else:
+            self.logger.info(f"Service '{service_name}' recreated and started successfully.")
+            if stdout: # Log stdout for 'up' even on success as it can be informative
+                self.logger.debug(f"Output from 'docker compose up':\n{stdout}")
+            return True
+
     def build_service(self, service_name: str, non_interactive=False) -> bool:
-        """Builds a specific docker-compose service."""
-        self.logger.info(f"Building service '{service_name}'...")
-        # Add --no-cache for now to ensure fresh builds, can be made configurable later
-        success, stdout, stderr = self._run_command(['docker', 'compose', 'build', '--no-cache', service_name])
+        """Builds a specific docker-compose service, utilizing Docker's build cache."""
+        self.logger.info(f"Building service '{service_name}' (using cache)...")
+        # Removed --no-cache to allow Docker to use its build cache.
+        # Docker's cache invalidation (e.g., for changed COPYed files) will trigger rebuilds as needed.
+        success, stdout, stderr = self._run_command(['docker', 'compose', 'build', service_name])
         if not success:
             self.logger.error(f"Failed to build service '{service_name}':\n{stderr}")
+            if stdout: # Build errors can sometimes appear in stdout
+                self.logger.error(f"Stdout:\n{stdout}")
             return False
         else:
             self.logger.info(f"Service '{service_name}' built successfully.")
+            if stdout: # Log stdout for 'build' as well, can be informative
+                self.logger.debug(f"Output from 'docker compose build':\n{stdout}")
             return True
 
     def get_ps(self) -> Optional[str]:
