@@ -11,10 +11,6 @@ DEFAULT_TOGETHER_MODEL = "Qwen/Qwen3-Coder-480B-A35B-Instruct-FP8"
 TOGETHER_API_ENDPOINT = "https://api.together.xyz/v1/chat/completions"
 TOGETHER_API_KEY_ENV_VAR = "TOGETHER_API_KEY"
 
-# Conservative default max tokens limit
-DEFAULT_MAX_TOKENS = 4096
-
-
 class TogetherAIClient(LLMClient):
     """
     Client for interacting with the Together.ai API.
@@ -104,9 +100,6 @@ class TogetherAIClient(LLMClient):
         payload = {
             "model": self.model,
             "messages": formatted_messages,
-            "max_tokens": DEFAULT_MAX_TOKENS,
-            # "temperature": 0.7,  # Optional parameters
-            # "top_p": 0.9,
         }
 
         try:
